@@ -39,6 +39,7 @@ void buffer_array_reset(buffer_array *b);
 buffer *buffer_array_append_get_buffer(buffer_array *b);
 
 buffer* buffer_init(void);
+buffer* buffer_init_buffer(buffer *b);
 buffer* buffer_init_string(const char *str);
 void buffer_free(buffer *b);
 void buffer_reset(buffer *b);
@@ -83,13 +84,16 @@ int buffer_is_equal_string(buffer *a, const char *s, size_t b_len);
 int buffer_caseless_compare(const char *a, size_t a_len, const char *b, size_t b_len);
 
 int buffer_append_string_hex(buffer *b, const char *in, size_t in_len);
-int buffer_append_string_url_encoded(buffer *b, const char *s);
-int buffer_append_string_html_encoded(buffer *b, const char *s);
+int buffer_append_string_url_encoded(buffer *b, const char *s, size_t s_len);
+int buffer_append_string_html_encoded(buffer *b, const char *s, size_t s_len);
 
 int buffer_urldecode_path(buffer *url);
 int buffer_urldecode_query(buffer *url);
 int buffer_path_simplify(buffer *dest, buffer *src);
 
+int buffer_to_lower(buffer *b);
+int buffer_to_upper(buffer *b);
+
 /** deprecated */
 int ltostr(char *buf, long val);
 char hex2int(unsigned char c);
@@ -113,7 +117,7 @@ int light_isalnum(int c);
 #define CONST_BUF_LEN(x) x->ptr, x->used - 1
 
 
-#define SEGFAULT() abort()
+#define SEGFAULT() do { fprintf(stderr, "%s.%d: aborted\n", __FILE__, __LINE__); abort(); } while(0)
 #define UNUSED(x) ( (void)(x) )
 
 #endif
