@@ -17,12 +17,19 @@ static keyvalue http_methods[] = {
 	{ HTTP_METHOD_HEAD, "HEAD" },
 	{ HTTP_METHOD_PROPFIND, "PROPFIND" },
 	{ HTTP_METHOD_PROPPATCH, "PROPPATCH" },
+	{ HTTP_METHOD_REPORT, "REPORT" },
 	{ HTTP_METHOD_OPTIONS, "OPTIONS" },
 	{ HTTP_METHOD_MKCOL, "MKCOL" },
 	{ HTTP_METHOD_PUT, "PUT" },
 	{ HTTP_METHOD_DELETE, "DELETE" },
 	{ HTTP_METHOD_COPY, "COPY" },
 	{ HTTP_METHOD_MOVE, "MOVE" },
+	{ HTTP_METHOD_LABEL, "LABEL" },
+	{ HTTP_METHOD_CHECKOUT, "CHECKOUT" },
+	{ HTTP_METHOD_CHECKIN, "CHECKIN" },
+	{ HTTP_METHOD_UNCHECKOUT, "UNCHECKOUT" },
+	{ HTTP_METHOD_VERSION_CONTROL, "VERSION-CONTROL" },
+
 	{ HTTP_METHOD_UNSET, NULL }
 };
 
