@@ -1,13 +1,35 @@
 #ifndef _KEY_VALUE_H_
 #define _KEY_VALUE_H_
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #ifdef HAVE_PCRE_H
 # include <pcre.h>
 #endif
 
-typedef enum { HTTP_METHOD_UNSET = -1, HTTP_METHOD_GET, HTTP_METHOD_POST, HTTP_METHOD_HEAD, HTTP_METHOD_PROPFIND, HTTP_METHOD_OPTIONS, HTTP_METHOD_MKCOL, HTTP_METHOD_PUT, HTTP_METHOD_DELETE, HTTP_METHOD_COPY, HTTP_METHOD_MOVE, HTTP_METHOD_PROPPATCH } http_method_t;
+typedef enum { 
+	HTTP_METHOD_UNSET = -1, 
+	HTTP_METHOD_GET, 
+	HTTP_METHOD_POST, 
+	HTTP_METHOD_HEAD, 
+	HTTP_METHOD_OPTIONS, 
+	HTTP_METHOD_PROPFIND,  /* WebDAV */
+	HTTP_METHOD_MKCOL, 
+	HTTP_METHOD_PUT, 
+	HTTP_METHOD_DELETE, 
+	HTTP_METHOD_COPY, 
+	HTTP_METHOD_MOVE, 
+	HTTP_METHOD_PROPPATCH, 
+	HTTP_METHOD_REPORT, /* DeltaV */
+	HTTP_METHOD_CHECKOUT,
+	HTTP_METHOD_CHECKIN,
+	HTTP_METHOD_VERSION_CONTROL,
+	HTTP_METHOD_UNCHECKOUT,
+	HTTP_METHOD_LABEL
+} http_method_t;
+
 typedef enum { HTTP_VERSION_UNSET = -1, HTTP_VERSION_1_0, HTTP_VERSION_1_1 } http_version_t;
 
 typedef struct {
