@@ -135,16 +135,20 @@ SETDEFAULTS_FUNC(mod_cml_set_defaults) {
 
 #define PATCH(x) \
 	p->conf.x = s->x;
-static int mod_cml_patch_connection(server *srv, connection *con, plugin_data *p, const char *stage, size_t stage_len) {
+static int mod_cml_patch_connection(server *srv, connection *con, plugin_data *p) {
 	size_t i, j;
+	plugin_config *s = p->config_storage[0];
+	
+	PATCH(ext);
+#if defined(HAVE_MEMCACHE_H)
+	PATCH(mc);
+#endif
+	PATCH(mc_namespace);
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
-		plugin_config *s = p->config_storage[i];
-		
-		/* not our stage */
-		if (!buffer_is_equal_string(dc->comp_key, stage, stage_len)) continue;
+		s = p->config_storage[i];
 		
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
@@ -167,20 +171,6 @@ static int mod_cml_patch_connection(server *srv, connection *con, plugin_data *p
 	
 	return 0;
 }
-
-static int mod_cml_setup_connection(server *srv, connection *con, plugin_data *p) {
-	plugin_config *s = p->config_storage[0];
-	UNUSED(srv);
-	UNUSED(con);
-		
-	PATCH(ext);
-#if defined(HAVE_MEMCACHE_H)
-	PATCH(mc);
-#endif
-	PATCH(mc_namespace);
-	
-	return 0;
-}
 #undef PATCH
 
 
@@ -313,17 +303,11 @@ URIHANDLER_FUNC(mod_cml_is_handled) {
 	char *c;
 	buffer *fn = con->physical.path;
 	plugin_data *p = p_d;
-	size_t i;
 	int ret;
 	
 	if (fn->used == 0) return HANDLER_ERROR;
 	
-	mod_cml_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		mod_cml_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
+	mod_cml_patch_connection(srv, con, p);
 	
 	buffer_reset(p->basedir);
 	buffer_reset(p->baseurl);
